@@ -57,34 +57,20 @@ Input format:
      Note that mutiple TaxIds pointing to the same accession are listed as
      comma-seperated integers. 
 
-Attentions:
-  1. Names should be distinct in taxa of different ranks.
-     But for these missing some taxon nodes, using names of parent nodes is allowed:
-
-       GB_GCA_018897955.1      d__Archaea;p__EX4484-52;c__EX4484-52;o__EX4484-52;f__LFW-46;g__LFW-46;s__LFW-46 sp018897155
-
-     It can also detect duplicate names with different ranks, e.g.,
-     the Class and Genus have the same name B47-G6, and the Order and Family
-     between them have different names. In this case, we reassign a new TaxId
-     by increasing the TaxId until it being distinct.
-
-       GB_GCA_003663585.1      d__Archaea;p__Thermoplasmatota;c__B47-G6;o__B47-G6B;f__47-G6;g__B47-G6;s__B47-G6 sp003663585
-
-  2. Taxa from different parents may have the same name.
-     We will assign different TaxIds to them. 
-
-     E.g., in ICTV, many viruses from different species have the same names.
-     In practice, we set the "Virus names(s)" as a subspecies rank and also
-     specify it as the accession.
-
-       Species             Virus name(s)
-       Jerseyvirus SETP3   Salmonella phage SETP7
-       Jerseyvirus SETP7   Salmonella phage SETP7
-
-  3. The generated TaxIds are not consecutive numbers, however some tools like MMSeqs2
+Attention:
+  1. Duplicated taxon names wit different ranks are allowed since v0.16.0, since
+     the rank and taxon name are contatenated for generating the TaxId.
+  2. The generated TaxIds are not consecutive numbers, however some tools like MMSeqs2
      required this, you can use the script below for convertion:
      
-     https://github.com/apcamargo/ictv-mmseqs2-protein-database/blob/master/fix_taxdump.py
+     https://github.com/apcamargo/ictv-mmseqs2-protein-database/blob/master/scripts/fix_taxdump.py
+
+  3. We only check and eliminate taxid collision within a single version of taxonomy data.
+     Therefore, if you create taxid-changelog with "taxid-changelog", different taxons
+     in multiple versions might have the same TaxIds and some change events might be wrong.
+
+     So a single version of taxonomic data created by "taxonkit create-taxdump" has no problem,
+     it's just the changelog might not be perfect.
 
 `,
 	Run: func(cmd *cobra.Command, args []string) {
@@ -272,6 +258,8 @@ Attentions:
 		ranks := make(map[uint32]uint8, 1<<16)
 
 		// child -> name
+		names0 := make(map[uint32]string, 1<<16)
+		// child -> rank+name
 		names := make(map[uint32]string, 1<<16)
 
 		// accession -> taxid
@@ -288,7 +276,6 @@ Attentions:
 			var first bool
 			var taxid uint32
 			var _name string
-			var _rank uint8
 			var ok bool
 			var reAssignTaxid bool
 
@@ -338,7 +325,7 @@ Attentions:
 								rankNames = rankNames[:0]
 								for i, val := range items0 {
 									if i+1 != fAccession {
-										rankNames = append(rankNames, val)
+										rankNames = append(rankNames, strings.ToLower(val))
 									} else {
 										rankNames = append(rankNames, "subspecies")
 									}
@@ -353,14 +340,17 @@ Attentions:
 								rankNames = rankNames[:0]
 								for i, val := range items0 {
 									if i+1 != fAccession {
-										rankNames = append(rankNames, val)
+										rankNames = append(rankNames, strings.ToLower(val))
 									}
 								}
 							}
 						} else {
 							numRanks = numFields
 
-							rankNames = items0
+							rankNames = make([]string, len(items0))
+							for i, val := range items0 {
+								rankNames[i] = strings.ToLower(val)
+							}
 						}
 
 						_items := make([]string, numFields)
@@ -484,13 +474,7 @@ Attentions:
 							continue
 						}
 
-						if j == 0 {
-							t.TaxIds[0] = uint32(xxhash.Sum64String(strings.ToLower(t.Names[0])) & 2147483647)
-							continue
-						}
-						if t.Names[j] != t.Names[j-1] {
-							t.TaxIds[j] = uint32(xxhash.Sum64String(strings.ToLower(t.Names[j])) & 2147483647)
-						}
+						t.TaxIds[j] = uint32(xxhash.Sum64String(rankNames[j]+strings.ToLower(t.Names[j])) & 2147483647)
 					}
 				} else {
 					// var ok bool
@@ -525,13 +509,7 @@ Attentions:
 							continue
 						}
 
-						if j == 0 {
-							t.TaxIds[0] = uint32(xxhash.Sum64String(strings.ToLower(t.Names[0])) & 2147483647)
-							continue
-						}
-						if t.Names[j] != t.Names[j-1] {
-							t.TaxIds[j] = uint32(xxhash.Sum64String(strings.ToLower(t.Names[j])) & 2147483647)
-						}
+						t.TaxIds[j] = uint32(xxhash.Sum64String(rankNames[j]+strings.ToLower(t.Names[j])) & 2147483647)
 					}
 				}
 
@@ -553,9 +531,9 @@ Attentions:
 					reAssignTaxid = false
 
 					if _name, ok = names[taxid]; ok { // check name
-						if _name != t.Names[i] { // two names hashed to the same uint32
+						if _name != rankNames[i]+t.Names[i] { // two names hashed to the same uint32
 							if config.Verbose {
-								log.Infof(`"%s" and "%s" having the same taxId: %d`, _name, t.Names[i], taxid)
+								log.Infof(`"%s" and "%s" have the same taxId: %d`, _name, rankNames[i]+t.Names[i], taxid)
 							}
 							reAssignTaxid = true
 						} else if i > 0 { // taxa with different parents may have the same names, many cases in ICTV
@@ -565,7 +543,7 @@ Attentions:
 								}
 							}
 							if j >= 0 { // have a non-root parent
-								if tree[taxid] != t.TaxIds[j] && names[tree[taxid]] != t.Names[j] { // not the same parent
+								if tree[taxid] != t.TaxIds[j] && names[tree[taxid]] != rankNames[j]+t.Names[j] { // not the same parent
 									if config.Verbose {
 										log.Infof(`"%s" (%d) and "%s" (%d) having the same child: %s`,
 											names[tree[taxid]], tree[taxid], t.Names[j], t.TaxIds[j], _name)
@@ -575,24 +553,17 @@ Attentions:
 							}
 						}
 					} else {
-						names[taxid] = t.Names[i]
+						names[taxid] = rankNames[i] + t.Names[i]
+						names0[taxid] = t.Names[i]
 					}
 
-					if _rank, ok = ranks[taxid]; ok {
-						if int(_rank) != i {
-							if config.Verbose {
-								// log.Debug(t)
-								log.Infof(`duplicate name (%s) with different ranks: "%s" and "%s"`, t.Names[i], rankNames[_rank], rankNames[i])
-							}
-							reAssignTaxid = true
-						}
-					} else {
+					if _, ok = ranks[taxid]; !ok {
 						ranks[taxid] = uint8(i)
 					}
 
 					if reAssignTaxid {
 						if config.Verbose {
-							log.Infof(`assign a new TaxId for "%s" (rank: %s): %d -> %d`, names[taxid], rankNames[i], taxid, taxid+1)
+							log.Infof(`  assign a new TaxId for "%s" (rank: %s): %d -> %d`, names[taxid], rankNames[i], taxid, taxid+1)
 						}
 						taxid++
 						t.TaxIds[i] = taxid
@@ -706,7 +677,7 @@ Attentions:
 
 		fmt.Fprintf(outfhNames, "%d\t|\t%s\t|\t\t|\tscientific name\t|\n", 1, "root")
 		for _, child := range taxids {
-			fmt.Fprintf(outfhNames, "%d\t|\t%s\t|\t\t|\tscientific name\t|\n", child, names[child])
+			fmt.Fprintf(outfhNames, "%d\t|\t%s\t|\t\t|\tscientific name\t|\n", child, names0[child])
 		}
 		log.Infof("%d records saved to %s", len(names)+1, fileNames)
 
@@ -921,12 +892,12 @@ func init() {
 	// --------------
 
 	createTaxDumpCmd.Flags().StringSliceP("null", "", []string{"", "NULL", "NA"}, "null value of taxa")
-	createTaxDumpCmd.Flags().StringSliceP("rank-names", "R", []string{}, "names of all ranks, leave it empty to use the first row of input as rank names")
+	createTaxDumpCmd.Flags().StringSliceP("rank-names", "R", []string{}, "names of all ranks, leave it empty to use the (lowercase) first row of input as rank names")
 
 	// --------------
 
 	createTaxDumpCmd.Flags().StringP("out-dir", "O", "", `output directory`)
-	createTaxDumpCmd.Flags().BoolP("force", "", false, `overwrite existed output directory`)
+	createTaxDumpCmd.Flags().BoolP("force", "", false, `overwrite existing output directory`)
 
 	// --------------
 
