@@ -37,6 +37,12 @@ var flineageCmd = &cobra.Command{
 	Short: "Reformat lineage in canonical ranks",
 	Long: `Reformat lineage in canonical ranks
 
+Warning:
+
+  - 'taxonkit reformat2' is recommended since Match 2025 when NCBI made
+    big changes to ranks.
+    See more: https://ncbiinsights.ncbi.nlm.nih.gov/2025/02/27/new-ranks-ncbi-taxonomy/
+
 Input:
 
   - List of TaxIds or lineages, one record per line.
@@ -61,6 +67,10 @@ Ambiguous names:
 
 Output format can be formated by flag --format, available placeholders:
 
+    {C}: cellular root
+    {a}: acellular root
+    {r}: realm
+    {d}: domain
     {k}: superkingdom
     {K}: kingdom
     {p}: phylum
@@ -89,6 +99,8 @@ Output format can contains some escape charactors like "\t".
 		delimiter := getFlagString(cmd, "delimiter")
 		blank := getFlagString(cmd, "miss-rank-repl")
 		prefix := getFlagString(cmd, "miss-rank-repl-prefix")
+		suffix := getFlagString(cmd, "miss-rank-repl-suffix")
+
 		iblank := getFlagString(cmd, "miss-taxid-repl")
 		fill := getFlagBool(cmd, "fill-miss-rank")
 		pseudoStrain := getFlagBool(cmd, "pseudo-strain")
@@ -114,6 +126,10 @@ Output format can contains some escape charactors like "\t".
 		printLineageInTaxid := getFlagBool(cmd, "show-lineage-taxids")
 
 		addPrefix := getFlagBool(cmd, "add-prefix")
+		prefixR := getFlagString(cmd, "prefix-r")
+		prefixC2 := getFlagString(cmd, "prefix-C")
+		prefixD := getFlagString(cmd, "prefix-d")
+		prefixA := getFlagString(cmd, "prefix-a")
 		prefixK := getFlagString(cmd, "prefix-k")
 		prefixK2 := getFlagString(cmd, "prefix-K")
 		prefixP := getFlagString(cmd, "prefix-p")
@@ -129,6 +145,10 @@ Output format can contains some escape charactors like "\t".
 		trim := getFlagBool(cmd, "trim")
 
 		prefixes := map[string]string{
+			"C": prefixC2,
+			"a": prefixA,
+			"d": prefixD,
+			"r": prefixR,
 			"k": prefixK,
 			"K": prefixK2,
 			"p": prefixP,
@@ -211,12 +231,8 @@ Output format can contains some escape charactors like "\t".
 
 		blankS := format
 		iblankS := format
-		for _, re := range reRankPlaceHolders {
-			blankS = re.ReplaceAllString(blankS, blank)
-		}
-		for _, re := range reRankPlaceHolders {
-			iblankS = re.ReplaceAllString(iblankS, iblank)
-		}
+		blankS = reRankPlaceHolder.ReplaceAllString(blankS, blank)
+		iblankS = reRankPlaceHolder.ReplaceAllString(iblankS, iblank)
 
 		fn := func(line string) (interface{}, bool, error) {
 			if len(line) == 0 || line[0] == '#' {
@@ -304,35 +320,54 @@ Output format can contains some escape charactors like "\t".
 					name = strings.ToLower(names[n-1])  // name
 					pname = strings.ToLower(names[n-2]) // parent name
 					var tmp map[string]uint32
+					found := true
 
 					if tmp, ok = name2parent2taxid[name]; !ok {
-						log.Warningf(`failed to query the TaxIds for: %s. Possible reasons: `, data[field])
-						log.Warningf(`  1) the lineage were produced with different taxonomy data files, please re-run taxonkit lineage;`)
-						log.Warningf(`  2) some taxon names contain delimiter (%s), please re-run taxonkit lineage and taxonkit reformat with different flag value of -d, e.g., -d "/"`, delimiter)
-						return line2flineage{line, "", ""}, true, nil
+						found = false
 					}
-
 					if taxid, ok = tmp[pname]; !ok {
-						log.Warningf(`failed to query the TaxIds for: %s. Possible reasons: `, data[field])
-						log.Warningf(`  1) the lineage were produced with different taxonomy data files, please re-run taxonkit lineage;`)
-						log.Warningf(`  2) some taxon names contain delimiter (%s), please re-run taxonkit lineage and taxonkit reformat with different flag value of -d, e.g., -d "/"`, delimiter)
-						return line2flineage{line, "", ""}, true, nil
+						found = false
 					}
-
-					// for cases where child-parent pairs are shared by multiple taxids.
-					pair := name + "__" + pname
-					var _ambids []uint32
-					if _ambids, ok = ambigous[pair]; ok {
-						tmp := make([]string, len(_ambids))
-						for _i, _taxid := range _ambids {
-							tmp[_i] = strconv.Itoa(int(_taxid))
-						}
-						log.Warningf("we can't distinguish the TaxIds (%s) for lineage: %s. But you can use -a/--output-ambiguous-result to return one possible result",
-							strings.Join(tmp, ", "), data[field])
-
-						if !outputAmbigous {
+					if !found { // try use the last node
+						// direct query via name2taxids
+						_taxids := name2taxids[strings.ToLower(names[n-1])]
+						if _taxids == nil {
+							log.Warningf(`failed to query the TaxId of: %s. Possible reasons: `, data[field])
+							log.Warningf(`  1) the lineage were produced with different taxonomy data files, please re-run taxonkit lineage;`)
+							log.Warningf(`  2) some taxon names contain delimiter (%s), please re-run taxonkit lineage and taxonkit reformat with different flag value of -d, e.g., -d "/"`, delimiter)
 							return line2flineage{line, "", ""}, true, nil
 						}
+
+						if len(*_taxids) == 1 { // found
+							taxid = (*_taxids)[0]
+						} else { // ambiguous name
+							tmp := make([]string, len(*_taxids))
+							for _i, _taxid := range *_taxids {
+								tmp[_i] = strconv.Itoa(int(_taxid))
+							}
+							log.Warningf(`we can't distinguish the TaxIds (%s) for lineage: %s. But you can use -a/--output-ambiguous-result to return one possible result`,
+								strings.Join(tmp, ", "), data[field])
+
+							if !outputAmbigous {
+								return line2flineage{line, "", ""}, true, nil
+							}
+						}
+					} else {
+						// for cases where child-parent pairs are shared by multiple taxids.
+						pair := name + "__" + pname
+						var _ambids []uint32
+						if _ambids, ok = ambigous[pair]; ok {
+							tmp := make([]string, len(_ambids))
+							for _i, _taxid := range _ambids {
+								tmp[_i] = strconv.Itoa(int(_taxid))
+							}
+							log.Warningf("we can't distinguish the TaxIds (%s) for lineage: %s. But you can use -a/--output-ambiguous-result to return one possible result",
+								strings.Join(tmp, ", "), data[field])
+
+							if !outputAmbigous {
+								return line2flineage{line, "", ""}, true, nil
+							}
+						}
 					}
 				}
 			}
@@ -362,13 +397,6 @@ Output format can contains some escape charactors like "\t".
 				ireplacements = make(map[string]string, len(matches))
 			}
 
-			for _, match := range matches {
-				replacements[match[1]] = blank
-				if printLineageInTaxid {
-					ireplacements[match[1]] = iblank
-				}
-			}
-
 			for i, name := range names {
 				rank = ranks[i]
 				taxid = taxids[i]
@@ -435,8 +463,12 @@ Output format can contains some escape charactors like "\t".
 						}
 					}
 
-					replacements[srank] = prefix + names[lastI] + " " + symbol2rank[srank]
-					// replacements[srank] = fmt.Sprintf("%s%s %s", prefix, names[lastI], symbol2rank[srank])
+					if suffix == "rank" {
+						replacements[srank] = prefix + names[lastI] + " " + symbol2rank[srank]
+						// replacements[srank] = fmt.Sprintf("%s%s %s", prefix, names[lastI], symbol2rank[srank])
+					} else {
+						replacements[srank] = prefix + names[lastI] + suffix
+					}
 				}
 			}
 
@@ -489,14 +521,30 @@ Output format can contains some escape charactors like "\t".
 			}
 
 			for srank, re := range reRankPlaceHolders {
-				if addPrefix {
-					flineage = re.ReplaceAllString(flineage, prefixes[srank]+replacements[srank])
-				} else {
-					flineage = re.ReplaceAllString(flineage, replacements[srank])
-				}
+				if replacements[srank] == "" {
+					if trim && symbol2weight[srank] > maxRankWeight {
+						flineage = re.ReplaceAllString(flineage, "")
 
-				if printLineageInTaxid {
-					iflineage = re.ReplaceAllString(iflineage, ireplacements[srank])
+						if printLineageInTaxid {
+							iflineage = re.ReplaceAllString(iflineage, "")
+						}
+					} else {
+						flineage = re.ReplaceAllString(flineage, blank)
+
+						if printLineageInTaxid {
+							iflineage = re.ReplaceAllString(iflineage, iblank)
+						}
+					}
+				} else {
+					if addPrefix {
+						flineage = re.ReplaceAllString(flineage, prefixes[srank]+replacements[srank])
+					} else {
+						flineage = re.ReplaceAllString(flineage, replacements[srank])
+					}
+
+					if printLineageInTaxid {
+						iflineage = re.ReplaceAllString(iflineage, ireplacements[srank])
+					}
 				}
 			}
 
@@ -546,7 +594,8 @@ func init() {
 	flineageCmd.Flags().StringP("format", "f", "{k};{p};{c};{o};{f};{g};{s}", "output format, placeholders of rank are needed")
 	flineageCmd.Flags().StringP("delimiter", "d", ";", "field delimiter in input lineage")
 	flineageCmd.Flags().StringP("miss-rank-repl", "r", "", `replacement string for missing rank`)
-	flineageCmd.Flags().StringP("miss-rank-repl-prefix", "p", "unclassified ", `prefix for estimated taxon level`)
+	flineageCmd.Flags().StringP("miss-rank-repl-prefix", "p", "unclassified ", `prefix for estimated taxon names`)
+	flineageCmd.Flags().StringP("miss-rank-repl-suffix", "s", "rank", `suffix for estimated taxon names. "rank" for rank name, "" for no suffix`)
 	flineageCmd.Flags().StringP("miss-taxid-repl", "R", "", `replacement string for missing taxid`)
 
 	flineageCmd.Flags().BoolP("fill-miss-rank", "F", false, "fill missing rank with lineage information of the next higher rank")
@@ -558,6 +607,10 @@ func init() {
 	flineageCmd.Flags().BoolP("output-ambiguous-result", "a", false, `output one of the ambigous result`)
 
 	flineageCmd.Flags().BoolP("add-prefix", "P", false, `add prefixes for all ranks, single prefix for a rank is defined by flag --prefix-X`)
+	flineageCmd.Flags().StringP("prefix-r", "", "r__", `prefix for realm, used along with flag -P/--add-prefix`)
+	flineageCmd.Flags().StringP("prefix-d", "", "d__", `prefix for domain, used along with flag -P/--add-prefix`)
+	flineageCmd.Flags().StringP("prefix-a", "", "d__", `prefix for acellular root, used along with flag -P/--add-prefix`)
+	flineageCmd.Flags().StringP("prefix-C", "", "d__", `prefix for cellular root, used along with flag -P/--add-prefix`)
 	flineageCmd.Flags().StringP("prefix-k", "", "k__", `prefix for superkingdom, used along with flag -P/--add-prefix`)
 	flineageCmd.Flags().StringP("prefix-K", "", "K__", `prefix for kingdom, used along with flag -P/--add-prefix`)
 	flineageCmd.Flags().StringP("prefix-p", "", "p__", `prefix for phylum, used along with flag -P/--add-prefix`)
@@ -570,5 +623,5 @@ func init() {
 	flineageCmd.Flags().StringP("prefix-S", "", "S__", `prefix for subspecies, used along with flag -P/--add-prefix`)
 	flineageCmd.Flags().StringP("prefix-T", "", "T__", `prefix for strain, used along with flag -P/--add-prefix`)
 
-	flineageCmd.Flags().BoolP("trim", "T", false, "do not fill missing rank lower than current rank")
+	flineageCmd.Flags().BoolP("trim", "T", false, "do not fill or add prefix for missing rank lower than current rank")
 }
